#include <security/pam_appl.h>
#include <security/pam_modules.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <pwd.h>
#include <syslog.h>
#include <libintl.h>

#define DEFAULT_USER	"nobody"

/*
 *
 * pam_setitem module for pam_sm_authenticate.
 *
 * options -
 *
 *	debug
 *	rhost={rhost}		defaults to 127.0.0.1
 *
 */

int
pam_sm_authenticate(
	pam_handle_t		*pamh,
	int 			flags,
	int			argc,
	const char		**argv)
{
	char			*user=NULL;
	int 			retval = PAM_AUTH_ERR;
	int			debug = 0, i;
	char			rhost[64];

	strcpy(rhost, "127.0.0.1");

	for (i = 0; i < argc; i++) {
		if (strcmp(argv[i], "debug") == 0)
			debug = 1;
		else if (sscanf(argv[i], "rhost=%s", rhost) == 1) {
			/* nothing */;
		}
		else
			syslog(LOG_DEBUG, "illegal pam_setitem option %s", argv[i]);
	}

	if (debug)
		syslog(LOG_DEBUG, "pam_setitem: rhost forced to %s\n", rhost);

/* pam_permit from Linux-PAM */

    /*
     * authentication requires we know who the user wants to be
     */

    retval = pam_get_user(pamh, &user, NULL);
    if (retval != PAM_SUCCESS) {
        if (debug) syslog(LOG_DEBUG, "get user returned error: %s", pam_strerror(pamh,retval));
        return retval;
    }
    if (user == NULL || *user == '\0') {
        if (debug) syslog(LOG_DEBUG, ("username not known"));
        pam_set_item(pamh, PAM_USER, (const void *) DEFAULT_USER);
    }
    user = NULL;                                            /* clean up */


/* end pam_permit */

	retval = pam_set_item(pamh, PAM_RHOST, (void **) rhost);
	if (retval != PAM_SUCCESS)
		return (retval);

	return retval;
}

/* stubs from Linux-PAM pam_permit */

int pam_sm_setcred(pam_handle_t *pamh,int flags,int argc
                   ,const char **argv)
{
     return PAM_SUCCESS;
}

/* --- account management functions --- */

int pam_sm_acct_mgmt(pam_handle_t *pamh,int flags,int argc
                     ,const char **argv)
{
     return PAM_SUCCESS;
}

/* --- password management --- */

int pam_sm_chauthtok(pam_handle_t *pamh,int flags,int argc
                     ,const char **argv)
{
     return PAM_SUCCESS;
}

/* --- session management --- */

int pam_sm_open_session(pam_handle_t *pamh,int flags,int argc
                        ,const char **argv)
{
    return PAM_SUCCESS;
}

int pam_sm_close_session(pam_handle_t *pamh,int flags,int argc
                         ,const char **argv)
{
     return PAM_SUCCESS;
}

/* end of module definition */

